from enum import Enum


class OrderEvents(Enum):
    ORDER_PROPOSAL_REQUEST = "order_proposal_request"
    ORDER_PROPOSAL_AVAILABLE = "order_proposal_available"
    ORDER_PROPOSAL_ACCEPTED = "order_proposal_accepted"
    ORDER_SCHEDULE_REQUEST = "order_schedule_request"
